/*cloudy the main routine, this IS Cloudy, ret 0 normal exit, 1 error exit,
 * called by maincl when used as standalone program */
#include "cddefines.h"
#include "itercnt.h"
#include "zonecnt.h"
#include "prtheader.h"
#include "dopunch.h"
#include "badstart.h"
#include "setcon.h"
#include "prtcomment.h"
#include "called.h"
#include "noexec.h"
#include "lines.h"
#include "firstdr.h"
#include "createpoint.h"
#include "createdata.h"
#include "nextdr.h"
#include "converge.h"
#include "setabundances.h"
#include "warngs.h"
#include "parse.h"
#include "prtfinal.h"
#include "opacity.h"
#include "taulines.h"
#include "rt.h"
#include "assertresults.h"
#include "startendzone.h"
#include "startenditer.h"
#include "lgendfun.h"
#include "plot.h"
#include "radinc.h"
#include "checksanity.h"
#include "dustop.h"
#include "grain.h"
#include "grainvar.h"
#include "cloudy.h"

/* returns 1 if disaster strikes, 0 if everything appears ok */
int cloudy(void)
{
	int lgOK,
		/* will be used to remember why we stopped, 
		 * return error exit in some cases */
		lgBadEnd;

	/* 
	 * this is the main routine to drive the modules that compute a model
	 * when cloudy is used as a stand-alone program 
	 * the main program (maincl) calls cdInit then cdDrive
	 * this sub is called by cdDrive which returns upon exiting
	 *
	 * this routine uses the following variables:
	 *
	 * ZoneCnt.nzone 
	 * this is the zone number, and is incremented here
	 * logical function lgEndFun returns a true condition if NZONE reaches
	 * NEND(ITER), the limit to the number of zones in this iteration
	 * ZoneCnt.nzone is totally controlled in this subroutine
	 *
	 * IterCnt.iter 
	 * this is the iteration number, it is 1 for the first iteration
	 * and is incremented in this subroutine at the end of each iteration
	 *
	 * IterCnt.itermx
	 * this is the limiit to the number of iterations, and is entered
	 * by the user
	 * This routine returns when IterCnt.iter > IterCnt.itermx
	 */

	/* NZONE is zero while initial search for conditions takes place */
	ZoneCnt.nzone = 0;

	/* ITER is iteration number */
	IterCnt.iter = 1;

	/* flag for error exit */
	lgBadEnd = FALSE;

#	ifdef INCLUDE_OLD_GRAINS
	/* >>chng 00 Mar 10, moved call to OldStyleGrainBlockDataInit from cdInit, PvH */
	/* initialize some old style grain data before other grains are read in */
	OldStyleGrainBlockDataInit();
#	endif

	/* interpret input data */
	ParseCommands();

	/* create several data sets by reading in data files, 
	 * but only if this is first call */
	CreateData();

	/* fix pointers to ionization edges and frequency array
	 * calls HydroCreate
	 * this routine has internal check for whether ever called before,
	 * if only retuns if this is a later call of code */
	CreatePoint();

	/* fix abundances of elements */
	SetAbundances();

	/* set continuum normalization, continuum itself, and ionization stage limits */
	SetCon(&lgOK);

	/* print header */
	PrtHeader();

	/* this is an option to stop after printing header only */
	if( noexec.lgNoExec )
		return(0);

	/* this happens when disaster strikes in the inital setup */
	if( !lgOK )
	{
		BadStart();
		return(1);
	}

	/* guess some outward optical depths, set inward optical depths, 
	 * also calls RTMake so escap probs ok before printout of trace */
	RTOptDepthInit();

	/* generate inital set of opacities, but only if this is the first call 
	 * in this coreload */
	OpacityCreate();

	/* this checks that various parts of the code still work properly */
	CheckSanity();

	/* find the initial temperature, punt if initial conditions outside range of code,*/
	if( ConvInitTemp() )
	{
		BadStart();
		return(1);
	}

	/* set thickness of first zone */
	FirstDR();

	/* find thickness of next zone */
	NextDR();

	/* set up some zone variables, correct continuum for sphercity, 
	 * after return, radius is equal to the inner radius, not outer radius
	 * of this zone */
	StartZone("init");

	plot("FIRST");

	/* outer loop is over number of iterations*/
	while( TRUE )
	{
		StartIter();
		ZoneCnt.nzone = 0;

		/* loop over zones within cloud, lgEndFun checks that model not complete */
		while( !lgEndFun() )
		{
			++ZoneCnt.nzone;

			/* set variable dealing with new radius, in startendzone.c */
			StartZone("incr");

			/* converge the pressure-temperature-ionization solution */
			ConvPresTempEdenIoniz();

			/* use changes in opacity, temperature, ionization, to fix new dr */
			NextDR();

			/* generate diffuse emission from this zone, add to outward beams,
			 * ConOutInter is set here */
			RTDiffuse();

			/* do work associated with incrementing this radius, 
			 * total attenuation and dilution of radiation fields takes place here,
			 * reflected continuum defined here*/
			RadInc();

			/* increment optical depths */
			RTOptDepthIncre();

			/* >>chng 99 may 4, move lines here from after RTDiffuse */
			/* fill in emission line array, adds outward lines */
			/* >>chng 99 dec 29, moved to here from below RTOptDepthIncre, 
			 * lines adds lines to outward beam,
			 * and these are attenuated in RadInc */
			lines();

			/* now that optical depths have been incremented, do escape prob */
			RTMake(TRUE);

			/* possibly punch out some results from this zone */
			DoPunch("MIDL");

			/* do some things to finish out this zone */
			EndZone();
		}

		/* close out this iteration */
		EndIter();

		/* print out some comments, generate warning and cautions*/
		PrtComment();

		/* punch stuff only needed on last shot */
		DoPunch("LAST" );

		plot("SECND");

		/* print out the results */
		PrtFinal();

		/* this is the normal exit out, reach limit to number of iterations,
		 * or if code has set busted */
		if( IterCnt.iter > IterCnt.itermx || called.lgBusted )
			break;

		/* reset limiting optical depth variables */
		RTOptDepthReset();

		++IterCnt.iter;
		ZoneCnt.nzone = 0;

		/* reinitialize some variables to initial conditions at previous first zone
		 * routine in startenditer.c */
		RestartIter();

		/* now redo escape probabilities */
		RTMake(TRUE);

		StartZone("init");

		/* find new first temperature, punt if gas totally cold and molecular */
		if( ConvInitTemp() )
		{ 
			lgBadEnd = TRUE;
			break;
		}
	}

	ClosePunchFiles();

	/* check whether any asserts were present and failed.  
	 * return is true if ok, false if not.  routine also checks
	 * number of warnings and returns false if not ok */
	lgOK = lgCheckAsserts(ioQQQ);

	/* >>>chng 00 Mar 10, return memory allocated for grains, PvH */
	ReturnGrainBins();

	if( lgOK && !warngs.lgWarngs && !lgBadEnd)
	{
		/* no failed asserts or warnings */
		return(0);
	}
	else
	{
		/* there were failed asserts or warnings */
		return(1);
	}

}
