/*gbar1 compute g-bar collision strength using Mewe approximations */
#include "cddefines.h"
#include "phycon.h"
#include "gbar1.h"

void gbar1(double ex, 
	  float *g)
{
	double y;

#	ifdef DEBUG_FUN
	fputs( "<+>gbar1()\n", debug_fp );
#	endif

	/*	*written by Dima Verner
	 *
	 * Calculation of the effective Gaunt-factor by use of Mewe (1972, A&A 20, 215)
	 * fits for permitted transitions in ions MgII, CaII, FeII (delta n = 0)
	 * Input parameters: 
	 * ex - excitation energy in Ryd - now K
	 * te  - temperature in K
	 * Output parameter:
	 * g  - effective Gaunt factor
	 */

	/* y = ex*157813.7/te */
	y = ex/phycon.te;
	*g = (float)(0.6 + 0.28*(log(1.0+1.0/y) - 0.4/POW2(y + 1.0)));


#	ifdef DEBUG_FUN
	fputs( " <->gbar1()\n", debug_fp );
#	endif
	return;
}

