/*GrainRateDr called by nextdr to find grain heating rate dr */
#include "cddefines.h"
#include "showme.h"
#include "hydrogenic.h"
#include "opac.h"
#include "nh.h"
#include "elmton.h"
#include "rfield.h"
#include "heavy.h"
#include "tff.h"
#include "heat.h"
#include "grainratedr.h"

void GrainRateDr(double *grfreqm, 
  double *gropacm)
{
	long int i, 
	  iplow;
	double xMax;

#	ifdef DEBUG_FUN
	fputs( "<+>GrainRateDr()\n", debug_fp );
#	endif

	/* in all following changed from anu2 to anu  july 25 95
	 *
	 * find maximum continuum interaction rate */

	/* not every continuum extends beyond C edge-this whole loop can add to zero
	 * use total opacity here
	 * test is to put in fir continuum if free free heating is important */
	if( hydro.FreeFreeHeat/heat.htot > 0.05 )
	{
		/* this is pointer to energy where cloud free free optical depth is unity */
		iplow = tffCom.ntff;
	}
	else
	{
		/* do up to carbon photo edge if carbon is turned on */
		/* >>> chng 00 apr 07, add test for whether element is turned on */
		if( elmton.lgElmtOn[5] )
		{
			/* carbon is turned on, use carbon 1 edge */
			iplow = Heavy.ipHeavy[0][5];
		}
		else
		{
			/* carbon truned off, use hydrogen balmer continuum */
			iplow = nh.ipHn[ipH][IP2S];
		}

	}

	xMax = 0.;
	/* integrate up to H edge */
	for( i=iplow-1; i < Heavy.ipHeavy[0][0]; i++ )
	{
		if( rfield.anu[i]*rfield.flux[i]/rfield.widflx[i]*opac.opac[i] > xMax )
		{
			xMax = rfield.anu[i]*rfield.flux[i]/rfield.widflx[i]*
			  opac.opac[i];
			*grfreqm = rfield.anu[i];
			*gropacm = opac.opac[i];
		}
	}
	/* integrate up to heii edge if he is turned on,
	 * this logic will not make sense if grains on but he off, which in itself makes no sense*/
	if( elmton.lgElmtOn[1] )
	{
		for( i=Heavy.ipHeavy[0][0]-1; i < Heavy.ipHeavy[1][1]; i++ )
		{
			if( rfield.anu[i]*rfield.flux[i]/rfield.widflx[i]*opac.opac[i] > xMax )
			{
				xMax = rfield.anu[i]*rfield.flux[i]/rfield.widflx[i]*
				  opac.opac[i];
				*grfreqm = rfield.anu[i];
				*gropacm = opac.opac[i];
			}
		}
	}

	/*begin sanity check */
	assert( *gropacm > 0. && *grfreqm > 0. );
	/*end sanity check */

#	ifdef DEBUG_FUN
	fputs( " <->GrainRateDr()\n", debug_fp );
#	endif
	return;
}
